@@ -1,5 +1,5 @@
+import io
 import unittest
-import six
 
 import pyrtl
 from pyrtl.corecircuits import _basic_add
@@ -28,8 +28,8 @@ class TraceWithBasicOpsBase(unittest.TestCase):
         sim_trace = pyrtl.SimulationTrace(wires_to_track=wtt)
         sim = self.sim(tracer=sim_trace)
         for i in range(8):
-            sim.step({})
-        output = six.StringIO()
+            sim.step()
+        output = io.StringIO()
         sim_trace.print_trace(output, compact=True)
         self.assertEqual(output.getvalue(), correct_string)
 
@@ -155,7 +155,7 @@ class PrintTraceBase(unittest.TestCase):
                         "in1_probe 0 1 2 3 4\n"
                         "in2       5 4 3 2 1\n"
                         "out       5 5 5 5 5\n")
-        output = six.StringIO()
+        output = io.StringIO()
         sim_trace.print_trace(output)
         self.assertEqual(output.getvalue(), correct_outp)
 
@@ -173,7 +173,7 @@ class PrintTraceBase(unittest.TestCase):
                         "in1_probe     0   100  1000  1100 10000\n"
                         "in2       10100 10000  1100  1000   100\n"
                         "out       10100 10100 10100 10100 10100\n")
-        output = six.StringIO()
+        output = io.StringIO()
         sim_trace.print_trace(output, base=2)
         self.assertEqual(output.getvalue(), correct_outp)
 
@@ -191,7 +191,7 @@ class PrintTraceBase(unittest.TestCase):
                         "in1_probe  0  6 14 22 30\n"
                         "in2       36 30 22 14  6\n"
                         "out       36 36 36 36 36\n")
-        output = six.StringIO()
+        output = io.StringIO()
         sim_trace.print_trace(output, base=8)
         self.assertEqual(output.getvalue(), correct_outp)
 
@@ -209,7 +209,7 @@ class PrintTraceBase(unittest.TestCase):
                         "in1_probe   0   9  12  1b  24\n"
                         "in2        2d  24  1b  12   9\n"
                         "out         0 144 1e6 1e6 144\n")
-        output = six.StringIO()
+        output = io.StringIO()
         sim_trace.print_trace(output, base=16)
         self.assertEqual(output.getvalue(), correct_outp)
 
@@ -269,7 +269,7 @@ class SimWithSpecialWiresBase(unittest.TestCase):
                         "in3  40 38 36 34 32 30 28 26 24 22\n"
                         "out2  0  5 10 15 20 25 30 35 40 45\n"
                         "out3 41 39 37 35 33 31 29 27 25 23\n")
-        output = six.StringIO()
+        output = io.StringIO()
         sim_trace.print_trace(output)
         self.assertEqual(output.getvalue(), correct_outp)
 
@@ -302,7 +302,7 @@ class SimStepMultipleBase(unittest.TestCase):
         out2 <<= in1 | in2
         self.inputs = {
             'in1': [0, 1, 3, 15, 14],
-            'in2': [6, 6, 6, 6, 6],
+            'in2': '66666',  # When a string, assumes each input is a single digit integer
         }
 
     def test_step_multiple_nsteps_no_inputs(self):
@@ -318,7 +318,7 @@ class SimStepMultipleBase(unittest.TestCase):
 
         correct_output = ("--- Values in base 10 ---\n"
                           "b 0 1 2 3 4\n")
-        output = six.StringIO()
+        output = io.StringIO()
         sim_trace.print_trace(output)
         self.assertEqual(output.getvalue(), correct_output)
 
@@ -393,7 +393,7 @@ class SimStepMultipleBase(unittest.TestCase):
                           "in2   6  6  6  6  6\n"
                           "out1  7  8  6 10  9\n"
                           "out2  6  7  7 15 14\n")
-        output = six.StringIO()
+        output = io.StringIO()
         sim_trace.print_trace(output)
         self.assertEqual(output.getvalue(), correct_output)
 
@@ -412,7 +412,26 @@ class SimStepMultipleBase(unittest.TestCase):
                           "in2   6  6  6  6  6\n"
                           "out1  7  8  6 10  9\n"
                           "out2  6  7  7 15 14\n")
-        output = six.StringIO()
+        output = io.StringIO()
+        sim_trace.print_trace(output)
+        self.assertEqual(output.getvalue(), correct_output)
+
+    def test_step_multiple_dont_care_expected(self):
+        sim_trace = pyrtl.SimulationTrace()
+        sim = self.sim(tracer=sim_trace)
+
+        expected = {
+            'out1': [7, '?', 6, 10],
+            'out2': '6?7?',
+        }
+        sim.step_multiple(self.inputs, expected, nsteps=4)
+
+        correct_output = (" --- Values in base 10 ---\n"
+                          "in1   0  1  3 15\n"
+                          "in2   6  6  6  6\n"
+                          "out1  7  8  6 10\n"
+                          "out2  6  7  7 15\n")
+        output = io.StringIO()
         sim_trace.print_trace(output)
         self.assertEqual(output.getvalue(), correct_output)
 
@@ -431,7 +450,7 @@ class SimStepMultipleBase(unittest.TestCase):
                           "in2  6 6 6\n"
                           "out1 7 8 6\n"
                           "out2 6 7 7\n")
-        output = six.StringIO()
+        output = io.StringIO()
         sim_trace.print_trace(output)
         self.assertEqual(output.getvalue(), correct_output)
 
@@ -443,7 +462,7 @@ class SimStepMultipleBase(unittest.TestCase):
             'out1': [7, 9, 4, 10, 9],
             'out2': [6, 2, 7, 8, 14],
         }
-        output = six.StringIO()
+        output = io.StringIO()
         sim.step_multiple(self.inputs, expected, file=output, stop_after_first_error=True)
 
         # Test the output about unexpected values
@@ -459,7 +478,7 @@ class SimStepMultipleBase(unittest.TestCase):
                           "in2  6 6\n"
                           "out1 7 8\n"
                           "out2 6 7\n")
-        output = six.StringIO()
+        output = io.StringIO()
         sim_trace.print_trace(output)
         self.assertEqual(output.getvalue(), correct_output)
 
@@ -471,7 +490,7 @@ class SimStepMultipleBase(unittest.TestCase):
             'out1': [7, 9, 4, 10, 9],
             'out2': [6, 2, 7, 8, 14],
         }
-        output = six.StringIO()
+        output = io.StringIO()
         sim.step_multiple(self.inputs, expected, file=output)
 
         # Test the output about unexpected values
@@ -489,7 +508,7 @@ class SimStepMultipleBase(unittest.TestCase):
                           "in2   6  6  6  6  6\n"
                           "out1  7  8  6 10  9\n"
                           "out2  6  7  7 15 14\n")
-        output = six.StringIO()
+        output = io.StringIO()
         sim_trace.print_trace(output)
         self.assertEqual(output.getvalue(), correct_output)
 
@@ -512,11 +531,11 @@ class TraceWithAdderBase(unittest.TestCase):
 
         # step through 15 cycles
         for i in range(15):
-            sim.step({})
+            sim.step()
 
-        output = six.StringIO()
+        output = io.StringIO()
         sim_trace.print_trace(output, compact=True)
-        file = six.StringIO()
+        file = io.StringIO()
         sim_trace.render_trace(file=file)  # want to make sure the code at least runs
         self.assertEqual(output.getvalue(), 'o 012345670123456\n')
 
@@ -595,9 +614,9 @@ b110 o
 
         # step through 15 cycles
         for i in range(15):
-            sim.step({})
+            sim.step()
 
-        test_output = six.StringIO()
+        test_output = io.StringIO()
         sim_trace.print_vcd(test_output)
         self.assertEqual(self.VCD_OUTPUT, test_output.getvalue())
 
@@ -626,7 +645,7 @@ class SimTraceWithMuxBase(unittest.TestCase):
         for i in range(6):
             self.sim.step(input_signals[i])
 
-        output = six.StringIO()
+        output = io.StringIO()
         self.sim_trace.print_trace(output, compact=True)
         self.assertEqual(output.getvalue(), 'muxout 120120\n')
 
@@ -664,7 +683,7 @@ class MemBlockBase(unittest.TestCase):
             sim.step({self.read_addr1: signals[0], self.read_addr2: signals[1],
                       self.write_addr: signals[2], self.write_data: signals[3]})
 
-        output = six.StringIO()
+        output = io.StringIO()
         self.sim_trace.print_trace(output, compact=True)
         self.assertEqual(output.getvalue(), 'o1 05560\no2 00560\n')
 
@@ -685,7 +704,7 @@ class MemBlockBase(unittest.TestCase):
         for signal in input_signals:
             sim.step(signal)
 
-        output = six.StringIO()
+        output = io.StringIO()
         self.sim_trace.print_trace(output, compact=True)
         self.assertEqual(output.getvalue(), 'o1 0077653107\no2 0076452310\n')
 
@@ -703,7 +722,7 @@ class MemBlockBase(unittest.TestCase):
             sim.step({self.read_addr1: signals[0], self.read_addr2: signals[1],
                       self.write_addr: signals[2], self.write_data: signals[3]})
 
-        output = six.StringIO()
+        output = io.StringIO()
         self.sim_trace.print_trace(output, compact=True)
         self.assertEqual(output.getvalue(), 'o1 05560\no2 00560\n')
 
@@ -734,7 +753,7 @@ class MemBlockBase(unittest.TestCase):
                 self.write_data: 2 + i
             })
         # check consistency of memory_value_map assignment, insertion, and modification
-        self.assertEquals(sim.inspect_mem(self.mem1), {0: 0, 1: 2, 2: 3, 3: 3, 4: 4, 5: 5})
+        self.assertEqual(sim.inspect_mem(self.mem1), {0: 0, 1: 2, 2: 3, 3: 3, 4: 4, 5: 5})
 
     def test_mem_val_map_defaults(self):
         read_addr3 = pyrtl.Input(self.addrwidth)
@@ -752,7 +771,7 @@ class MemBlockBase(unittest.TestCase):
                 self.write_addr: 0,
                 self.write_data: 0
             })
-        output = six.StringIO()
+        output = io.StringIO()
         self.sim_trace.print_trace(output, compact=True)
         self.assertEqual(output.getvalue(), 'o1 000000\n'
                                             'o2 000000\n'
@@ -774,7 +793,7 @@ class MemBlockBase(unittest.TestCase):
                 self.write_addr: 0,
                 self.write_data: 0
             })
-        output = six.StringIO()
+        output = io.StringIO()
         self.sim_trace.print_trace(output, compact=True)
         self.assertEqual(output.getvalue(), 'o1 000000\n'
                                             'o2 000000\n'
@@ -815,7 +834,7 @@ class MemBlockLargeBase(unittest.TestCase):
             sim.step({self.read_addr1: signals[0], self.read_addr2: signals[1],
                       self.write_addr: signals[2], self.write_data: signals[3]})
 
-        output = six.StringIO()
+        output = io.StringIO()
         correct_outp = ("--- Values in base 10 ---\n"
                         "o1                    0 %d %d                    6                    0\n"
                         "o2                    0                    0 %d                    6                    0\n"  # noqa
@@ -829,7 +848,7 @@ class RegisterDefaultsBase(unittest.TestCase):
         pyrtl.reset_working_block()
         self.i = pyrtl.Input(bitwidth=3)
         self.r1 = pyrtl.Register(name='r1', bitwidth=3)
-        self.r2 = pyrtl.Register(name='r2', bitwidth=3)
+        self.r2 = pyrtl.Register(name='r2', bitwidth=3, reset_value=7)
         self.o = pyrtl.Output(name='o', bitwidth=3)
         self.r1.next <<= self.i
         self.r2.next <<= self.r1
@@ -841,21 +860,24 @@ class RegisterDefaultsBase(unittest.TestCase):
         sim = self.sim(tracer=sim_trace, **kwargs)
         for i in range(8):
             sim.step({self.i: i})
-        output = six.StringIO()
+        output = io.StringIO()
         sim_trace.print_trace(output, compact=True)
         self.assertEqual(output.getvalue(), correct_string)
 
-    def test_default_value(self):
-        self.check_trace('o 55012345\n', default_value=5)
+    def test_reset_value(self):
+        self.check_trace('o 70012345\n')
 
-    def test_register_map(self):
+    def test_register_map_overrides_reset_value(self):
         self.check_trace('o 36012345\n', register_value_map={self.r1: 6, self.r2: 3})
 
     def test_partial_map(self):
-        self.check_trace('o 06012345\n', register_value_map={self.r1: 6})
+        self.check_trace('o 76012345\n', register_value_map={self.r1: 6})
 
-    def test_map_and_default(self):
-        self.check_trace('o 56012345\n', default_value=5, register_value_map={self.r1: 6})
+    def test_partial_map_overrides_default(self):
+        self.check_trace('o 76012345\n', default_value=5, register_value_map={self.r1: 6})
+
+    def test_default_used_for_non_reset_value(self):
+        self.check_trace('o 75012345\n', default_value=5)
 
 
 class RomBlockSimBase(unittest.TestCase):
@@ -876,7 +898,7 @@ class RomBlockSimBase(unittest.TestCase):
         return out_string
 
     def compareIO(self, sim_trace_a, expected_output):
-        output = six.StringIO()
+        output = io.StringIO()
         sim_trace_a.print_trace(output, compact=True)
         self.assertEqual(output.getvalue(), expected_output)
 
@@ -1012,7 +1034,7 @@ class TraceErrorBase(unittest.TestCase):
         sim.step_multiple(provided_inputs={}, nsteps=10)
         with self.assertRaises(pyrtl.PyrtlError) as ex:
             sim.tracer.render_trace()
-        self.assertEquals(
+        self.assertEqual(
             str(ex.exception),
             "Empty trace list. This may have occurred because "
             "untraceable wires were removed prior to simulation, "
