@@ -1,6 +1,6 @@
-import unittest
-import six
+import enum
 import io
+import unittest
 
 import pyrtl
 from pyrtl.corecircuits import _basic_add
@@ -22,12 +22,11 @@ class TraceWithBasicOpsBase(unittest.TestCase):
         self.r = pyrtl.Register(bitwidth=self.bitwidth, name='r')
 
     def check_trace(self, correct_string):
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
         for i in range(8):
-            sim.step({})
-        output = six.StringIO()
-        sim_trace.print_trace(output, compact=True)
+            sim.step()
+        output = io.StringIO()
+        sim.tracer.print_trace(output, compact=True)
         self.assertEqual(output.getvalue(), correct_string)
 
     def test_not_simulation(self):
@@ -66,8 +65,7 @@ class TraceWithBasicOpsBase(unittest.TestCase):
         o = pyrtl.Output(name='o')
         o <<= i - 1
 
-        tracer = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=tracer)
+        sim = self.sim()
         sim.step({i: 1})
         self.assertEqual(sim.inspect(o), 0)
         sim.step({i: 0})
@@ -128,6 +126,23 @@ class TraceWithBasicOpsBase(unittest.TestCase):
         self.r2.next <<= self.r + pyrtl.Const(2, bitwidth=self.bitwidth)
         self.check_trace(' r 00224466\nr2 02244660\n')
 
+    def test_negative_input(self):
+        """Test negative Input values."""
+        pyrtl.reset_working_block()
+
+        bitwidth = 4
+        i = pyrtl.Input(bitwidth=bitwidth, name='i')
+        o = pyrtl.Output(bitwidth=bitwidth, name='o')
+        o <<= i
+
+        sim = self.sim()
+        for n in range(-(2 ** (bitwidth - 1)), 2 ** (bitwidth - 1)):
+            sim.step({'i': n})
+            inspected_val = sim.inspect('o')
+            actual_val = pyrtl.val_to_signed_integer(
+                inspected_val, bitwidth=bitwidth)
+            self.assertEqual(actual_val, n)
+
 
 class RenderTraceBase(unittest.TestCase):
     def setUp(self):
@@ -135,6 +150,8 @@ class RenderTraceBase(unittest.TestCase):
         a, b, c = pyrtl.input_list('a/8 b/8 c/1')
         o = pyrtl.Output()
         o <<= a + b - c
+        self.renderer = pyrtl.simulation.WaveRenderer(
+            pyrtl.simulation.AsciiRendererConstants)
 
     def check_rendered_trace(self, expected, **kwargs):
         sim = pyrtl.Simulation()
@@ -144,55 +161,132 @@ class RenderTraceBase(unittest.TestCase):
             'c': [0, 1, 1, 0, 1]
         })
         buff = io.StringIO()
-        sim.tracer.render_trace(file=buff, render_cls=pyrtl.simulation.AsciiWaveRenderer,
-                                extra_line=False, **kwargs)
+        sim.tracer.render_trace(file=buff, renderer=self.renderer, **kwargs)
         self.assertEqual(buff.getvalue(), expected)
 
     def test_hex_trace(self):
         expected = (
-            "  -0                       \n"
-            "a 0x1  x0x4 x0x9 x0xb x0xc \n"
-            "b 0x2  x0x17x0x2bx0x78x0x0 \n"
-            "c _____/---------\\____/----\n"  # escaped backslash
+            " |0   |1   |2   |3   |4   \n"
+            "  \n"
+            "a 0x1 |0x4 |0x9 |0xb |0xc \n"
+            "  \n"
+            "b 0x2 |0x17|0x2b|0x78|----\n"
+            "  \n"
+            "c ____,---------.____,----\n"
         )
         self.check_rendered_trace(expected)
 
     def test_oct_trace(self):
         expected = (
-            "  -0                            \n"
-            "a 0o1   x0o4  x0o11 x0o13 x0o14 \n"
-            "b 0o2   x0o27 x0o53 x0o170x0o0  \n"
-            "c ______/-----------\\_____/-----\n"  # escaped backslash
+            " |0    |1    |2    |3    |4    \n"
+            "  \n"
+            "a 0o1  |0o4  |0o11 |0o13 |0o14 \n"
+            "  \n"
+            "b 0o2  |0o27 |0o53 |0o170|0o0  \n"
+            "  \n"
+            "c _____,-----------._____,-----\n"
         )
-        self.check_rendered_trace(expected, repr_func=oct, symbol_len=None)
+
+        self.check_rendered_trace(expected, repr_func=oct)
 
     def test_bin_trace(self):
         expected = (
-            "  -0                                                \n"
-            "a 0b1       x0b100    x0b1001   x0b1011   x0b1100   \n"
-            "b 0b10      x0b10111  x0b101011 x0b1111000x0b0      \n"
-            "c __________/-------------------\\_________/---------\n"  # escaped backslash
+            " |0        |1        |2        |3        |4        \n"
+            "  \n"
+            "a 0b1      |0b100    |0b1001   |0b1011   |0b1100   \n"
+            "  \n"
+            "b 0b10     |0b10111  |0b101011 |0b1111000|0b0      \n"
+            "  \n"
+            "c _________,-------------------._________,---------\n"
         )
-        self.check_rendered_trace(expected, repr_func=bin, symbol_len=None)
+
+        self.check_rendered_trace(expected, repr_func=bin)
 
     def test_decimal_trace(self):
         expected = (
-            "  -0                  \n"
-            "a 1   x4  x9  x11 x12 \n"
-            "b 2   x23 x43 x120x0  \n"
-            "c ____/-------\\___/---\n"  # escaped backslash
+            " |0  |1  |2  |3  |4  \n"
+            "  \n"
+            "a 1  |4  |9  |11 |12 \n"
+            "  \n"
+            "b 2  |23 |43 |120|0  \n"
+            "  \n"
+            "c ___,-------.___,---\n"
         )
-        self.check_rendered_trace(expected, repr_func=str, symbol_len=None)
+        self.check_rendered_trace(expected, repr_func=str)
+
+
+class RenderTraceBase(unittest.TestCase):
+    def setUp(self):
+        pyrtl.reset_working_block()
+        a = pyrtl.Input(name='a', bitwidth=4)
+        o = pyrtl.Output()
+        o <<= a
+        self.renderer = pyrtl.simulation.WaveRenderer(
+            pyrtl.simulation.AsciiRendererConstants)
+
+    def check_rendered_trace(self, expected, **kwargs):
+        sim = pyrtl.Simulation()
+        sim.step_multiple({'a': list(reversed(range(10))) * 2})
+        buff = io.StringIO()
+        sim.tracer.render_trace(file=buff, renderer=self.renderer, **kwargs)
+        self.assertEqual(buff.getvalue(), expected)
+
+    def test_long_decimal_trace(self):
+        '''Check that long cycle names are truncated.
+
+        The most significant digits should be omitted.
+
+        '''
+        expected = (
+            " |0|1|2|3|4|5|6|7|8|9|0|1|2|3|4|5|6|7|8|9\n"
+            "  \n"
+            "a 9|8|7|6|5|4|3|2|1|-|9|8|7|6|5|4|3|2|1|-\n"
+        )
+        self.check_rendered_trace(expected, repr_func=int)
 
 
 class RenderTraceCustomBase(unittest.TestCase):
     def setUp(self):
         pyrtl.reset_working_block()
+        self.renderer = pyrtl.simulation.WaveRenderer(
+            pyrtl.simulation.AsciiRendererConstants)
+
+    def test_enum_name(self):
+        class State(enum.IntEnum):
+            FOO = 0
+            BAR = 1
+        state = pyrtl.Input(name='state', bitwidth=1)
+        sim = pyrtl.Simulation()
+        sim.step_multiple({'state': [State.FOO, State.BAR]})
+        buff = io.StringIO()
+        sim.tracer.render_trace(
+            file=buff, renderer=self.renderer,
+            repr_per_name={'state': pyrtl.enum_name(State)})
+        expected = (
+            "     |0  |1  \n"
+            "      \n"
+            "state FOO|BAR\n"
+        )
+        self.assertEqual(buff.getvalue(), expected)
+
+    def test_val_to_signed_integer(self):
+        bitwidth = 2
+        counter = pyrtl.Register(name='counter', bitwidth=bitwidth)
+        counter.next <<= counter + 1
+        sim = pyrtl.Simulation()
+        sim.step_multiple(nsteps=2 ** bitwidth)
+        buff = io.StringIO()
+        sim.tracer.render_trace(file=buff, renderer=self.renderer,
+                                repr_func=pyrtl.val_to_signed_integer)
+        expected = (
+            "       |0 |1 |2 |3 \n"
+            "        \n"
+            "counter --|1 |-2|-1\n"
+        )
+        self.assertEqual(buff.getvalue(), expected)
 
     def test_custom_repr_per_wire(self):
-        from enum import IntEnum
-
-        class Foo(IntEnum):
+        class Foo(enum.IntEnum):
             A = 0
             B = 1
             C = 2
@@ -218,13 +312,16 @@ class RenderTraceCustomBase(unittest.TestCase):
             'i': [1, 2, 4, 8, 0]
         })
         buff = io.StringIO()
-        sim.tracer.render_trace(file=buff, render_cls=pyrtl.simulation.AsciiWaveRenderer,
-                                extra_line=None, repr_per_name={'state': Foo}, symbol_len=None)
+        sim.tracer.render_trace(file=buff, renderer=self.renderer,
+                                repr_per_name={'state': pyrtl.enum_name(Foo)})
         expected = (
-            "      -0                            \n"
-            "    i 0x1   x0x2  x0x4  x0x8  x0x0  \n"
-            "    o 0x0         x0x1  x0x2  x0x3  \n"
-            "state Foo.A       xFoo.BxFoo.CxFoo.D\n"
+            "     |0  |1  |2  |3  |4  \n"
+            "      \n"
+            "    i 0x1|0x2|0x4|0x8|---\n"
+            "      \n"
+            "    o -------|0x1|0x2|0x3\n"
+            "      \n"
+            "state A      |B  |C  |D  \n"
         )
         self.assertEqual(buff.getvalue(), expected)
 
@@ -239,8 +336,7 @@ class PrintTraceBase(unittest.TestCase):
 
     def test_print_trace_single_dig_notcompact(self):
         self.out <<= pyrtl.probe(self.in1, "in1_probe") + self.in2
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
         for i in range(5):
             sim.step({
                 self.in1: i,
@@ -251,14 +347,13 @@ class PrintTraceBase(unittest.TestCase):
                         "in1_probe 0 1 2 3 4\n"
                         "in2       5 4 3 2 1\n"
                         "out       5 5 5 5 5\n")
-        output = six.StringIO()
-        sim_trace.print_trace(output)
+        output = io.StringIO()
+        sim.tracer.print_trace(output)
         self.assertEqual(output.getvalue(), correct_outp)
 
     def test_print_trace_base2(self):
         self.out <<= pyrtl.probe(self.in1, "in1_probe") + self.in2
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
         for i in range(5):
             sim.step({
                 self.in1: 4 * i,
@@ -269,14 +364,13 @@ class PrintTraceBase(unittest.TestCase):
                         "in1_probe     0   100  1000  1100 10000\n"
                         "in2       10100 10000  1100  1000   100\n"
                         "out       10100 10100 10100 10100 10100\n")
-        output = six.StringIO()
-        sim_trace.print_trace(output, base=2)
+        output = io.StringIO()
+        sim.tracer.print_trace(output, base=2)
         self.assertEqual(output.getvalue(), correct_outp)
 
     def test_print_trace_base8(self):
         self.out <<= pyrtl.probe(self.in1, "in1_probe") + self.in2
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
         for i in range(5):
             sim.step({
                 self.in1: 6 * i,
@@ -287,14 +381,13 @@ class PrintTraceBase(unittest.TestCase):
                         "in1_probe  0  6 14 22 30\n"
                         "in2       36 30 22 14  6\n"
                         "out       36 36 36 36 36\n")
-        output = six.StringIO()
-        sim_trace.print_trace(output, base=8)
+        output = io.StringIO()
+        sim.tracer.print_trace(output, base=8)
         self.assertEqual(output.getvalue(), correct_outp)
 
     def test_print_trace_base16(self):
         self.out <<= pyrtl.probe(self.in1, "in1_probe") * self.in2
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
         for i in range(5):
             sim.step({
                 self.in1: 9 * i,
@@ -305,11 +398,36 @@ class PrintTraceBase(unittest.TestCase):
                         "in1_probe   0   9  12  1b  24\n"
                         "in2        2d  24  1b  12   9\n"
                         "out         0 144 1e6 1e6 144\n")
-        output = six.StringIO()
-        sim_trace.print_trace(output, base=16)
+        output = io.StringIO()
+        sim.tracer.print_trace(output, base=16)
         self.assertEqual(output.getvalue(), correct_outp)
 
 
+class PrintPerfCountersBase(unittest.TestCase):
+    def setUp(self):
+        pyrtl.reset_working_block()
+        self.a = pyrtl.Input(bitwidth=1, name='a')
+        self.b = pyrtl.Input(bitwidth=1, name='b')
+        self.c = pyrtl.Input(bitwidth=1, name='c')
+
+    def test_print_perf_counters(self):
+        sim = self.sim()
+        for i in range(16):
+            sim.step({
+                self.a: i % 2 == 0,
+                self.b: i % 4 == 0,
+                self.c: i % 8 == 0,
+            })
+        output = io.StringIO()
+        sim.tracer.print_perf_counters('a', 'b', 'c', file=output)
+        # a is high for 8 cycles.
+        self.assertTrue('8' in output.getvalue())
+        # b is high for 4 cycles.
+        self.assertTrue('4' in output.getvalue())
+        # c is high for 2 cycles.
+        self.assertTrue('2' in output.getvalue())
+
+
 class SimWithSpecialWiresBase(unittest.TestCase):
     def setUp(self):
         pyrtl.reset_working_block()
@@ -357,8 +475,7 @@ class SimWithSpecialWiresBase(unittest.TestCase):
         out2 = pyrtl.Output(16, "out3")
         out1 <<= in1 + in2
         out2 <<= in3 | truth
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
         for i in range(10):
             sim.step({
                 'in1': 2 * i,
@@ -371,14 +488,12 @@ class SimWithSpecialWiresBase(unittest.TestCase):
                         "in3  40 38 36 34 32 30 28 26 24 22\n"
                         "out2  0  5 10 15 20 25 30 35 40 45\n"
                         "out3 41 39 37 35 33 31 29 27 25 23\n")
-        output = six.StringIO()
-        sim_trace.print_trace(output)
+        output = io.StringIO()
+        sim.tracer.print_trace(output)
         self.assertEqual(output.getvalue(), correct_outp)
 
     def test_consts_from_int_enums(self):
-        from enum import IntEnum
-
-        class MyEnum(IntEnum):
+        class MyEnum(enum.IntEnum):
             A = 0
             B = 1
             C = 3
@@ -406,8 +521,7 @@ class SimWithSpecialWiresBase(unittest.TestCase):
         c1 = pyrtl.Const(1, 1, "c1")
         out1 = pyrtl.Output(16, "out1")
         out1 <<= in1 + c1
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
         for i in range(10):
             sim.step({
                 'in1': 2 * i,
@@ -416,8 +530,8 @@ class SimWithSpecialWiresBase(unittest.TestCase):
                         "c1    1  1  1  1  1  1  1  1  1  1\n"
                         "in1   0  2  4  6  8 10 12 14 16 18\n"
                         "out1  1  3  5  7  9 11 13 15 17 19\n")
-        output = six.StringIO()
-        sim_trace.print_trace(output)
+        output = io.StringIO()
+        sim.tracer.print_trace(output)
         self.assertEqual(output.getvalue(), correct_outp)
 
     def test_reset_value_in_reg(self):
@@ -426,11 +540,10 @@ class SimWithSpecialWiresBase(unittest.TestCase):
         o = pyrtl.Output(4, 'o')
         o <<= r
 
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
         sim.step_multiple(nsteps=7)
-        output = six.StringIO()
-        sim_trace.print_trace(output, compact=True)
+        output = io.StringIO()
+        sim.tracer.print_trace(output, compact=True)
         self.assertEqual(output.getvalue(), 'o 2301230\n')
 
     def test_reset_value_overridden_in_simulation(self):
@@ -439,11 +552,10 @@ class SimWithSpecialWiresBase(unittest.TestCase):
         o = pyrtl.Output(4, 'o')
         o <<= r
 
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace, register_value_map={r: 1})
+        sim = self.sim(register_value_map={r: 1})
         sim.step_multiple(nsteps=7)
-        output = six.StringIO()
-        sim_trace.print_trace(output, compact=True)
+        output = io.StringIO()
+        sim.tracer.print_trace(output, compact=True)
         self.assertEqual(output.getvalue(), 'o 1230123\n')
 
     def test_default_value_for_registers_without_reset_value(self):
@@ -454,12 +566,11 @@ class SimWithSpecialWiresBase(unittest.TestCase):
         o = pyrtl.Output(4, 'o')
         o <<= r + s
 
-        sim_trace = pyrtl.SimulationTrace()
         # Should set default value for s only (since r has specified 'reset_value')
-        sim = self.sim(tracer=sim_trace, default_value=3)
+        sim = self.sim(default_value=3)
         sim.step_multiple(nsteps=7)
-        output = six.StringIO()
-        sim_trace.print_trace(output, compact=True)
+        output = io.StringIO()
+        sim.tracer.print_trace(output, compact=True)
         self.assertEqual(output.getvalue(), 'o 6222622\nr 3012301\ns 3210321\n')
 
 
@@ -472,8 +583,7 @@ class SimInputValidationBase(unittest.TestCase):
         i = pyrtl.Input(bitwidth=2, name='i')
         counter.next <<= counter + i
 
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
         for cycle in range(4):
             sim.step({i: cycle})
         with self.assertRaises(pyrtl.PyrtlError):
@@ -485,9 +595,6 @@ class SimInputValidationBase(unittest.TestCase):
         c = pyrtl.Output()
         c <<= a + b
 
-        with self.assertRaises(pyrtl.PyrtlError):
-            sim_trace = pyrtl.SimulationTrace()
-
 
 class SimStepAccessInternalMemoryBase(unittest.TestCase):
 
@@ -562,19 +669,17 @@ class SimStepMultipleBase(unittest.TestCase):
         a.next <<= a + 1
         b <<= a
 
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
         sim.step_multiple(nsteps=5)
 
         correct_output = ("--- Values in base 10 ---\n"
                           "b 0 1 2 3 4\n")
-        output = six.StringIO()
-        sim_trace.print_trace(output)
+        output = io.StringIO()
+        sim.tracer.print_trace(output)
         self.assertEqual(output.getvalue(), correct_output)
 
     def test_step_multiple_nsteps_gt_ninputs(self):
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
 
         with self.assertRaises(pyrtl.PyrtlError) as error:
             sim.step_multiple(self.inputs, nsteps=6)
@@ -585,8 +690,7 @@ class SimStepMultipleBase(unittest.TestCase):
         )
 
     def test_step_multiple_no_inputs(self):
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
 
         with self.assertRaises(pyrtl.PyrtlError) as error:
             sim.step_multiple()
@@ -595,8 +699,7 @@ class SimStepMultipleBase(unittest.TestCase):
                          'or a number of steps to simulate')
 
     def test_step_multiple_bad_nsteps1(self):
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
 
         with self.assertRaises(pyrtl.PyrtlError) as error:
             sim.step_multiple({'in1': [], 'in2': []})
@@ -604,8 +707,7 @@ class SimStepMultipleBase(unittest.TestCase):
                          'must simulate at least one step')
 
     def test_step_multiple_bad_nsteps2(self):
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
 
         with self.assertRaises(pyrtl.PyrtlError) as error:
             sim.step_multiple(self.inputs, nsteps=-1)
@@ -613,8 +715,7 @@ class SimStepMultipleBase(unittest.TestCase):
                          'must simulate at least one step')
 
     def test_step_multiple_no_values_for_each_step_of_input(self):
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
 
         with self.assertRaises(pyrtl.PyrtlError) as error:
             sim.step_multiple({'in1': [0, 1, 3], 'in2': [1]})
@@ -623,8 +724,7 @@ class SimStepMultipleBase(unittest.TestCase):
                          'for each step of simulation')
 
     def test_step_multiple_no_values_for_each_step_of_given_outputs(self):
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
 
         with self.assertRaises(pyrtl.PyrtlError) as error:
             sim.step_multiple(self.inputs,
@@ -635,8 +735,7 @@ class SimStepMultipleBase(unittest.TestCase):
                          'each step of simulation')
 
     def test_step_multiple_no_expected_check(self):
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
 
         sim.step_multiple(self.inputs)
 
@@ -645,13 +744,12 @@ class SimStepMultipleBase(unittest.TestCase):
                           "in2   6  6  6  6  6\n"
                           "out1  7  8  6 10  9\n"
                           "out2  6  7  7 15 14\n")
-        output = six.StringIO()
-        sim_trace.print_trace(output)
+        output = io.StringIO()
+        sim.tracer.print_trace(output)
         self.assertEqual(output.getvalue(), correct_output)
 
     def test_step_multiple_no_errors(self):
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
 
         expected = {
             'out1': [7, 8, 6, 10, 9],
@@ -664,13 +762,12 @@ class SimStepMultipleBase(unittest.TestCase):
                           "in2   6  6  6  6  6\n"
                           "out1  7  8  6 10  9\n"
                           "out2  6  7  7 15 14\n")
-        output = six.StringIO()
-        sim_trace.print_trace(output)
+        output = io.StringIO()
+        sim.tracer.print_trace(output)
         self.assertEqual(output.getvalue(), correct_output)
 
     def test_step_multiple_dont_care_expected(self):
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
 
         expected = {
             'out1': [7, '?', 6, 10],
@@ -683,13 +780,12 @@ class SimStepMultipleBase(unittest.TestCase):
                           "in2   6  6  6  6\n"
                           "out1  7  8  6 10\n"
                           "out2  6  7  7 15\n")
-        output = six.StringIO()
-        sim_trace.print_trace(output)
+        output = io.StringIO()
+        sim.tracer.print_trace(output)
         self.assertEqual(output.getvalue(), correct_output)
 
     def test_step_multiple_no_errors_nsteps_specified(self):
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
 
         expected = {
             'out1': [7, 8, 6, 10, 9],
@@ -702,19 +798,18 @@ class SimStepMultipleBase(unittest.TestCase):
                           "in2  6 6 6\n"
                           "out1 7 8 6\n"
                           "out2 6 7 7\n")
-        output = six.StringIO()
-        sim_trace.print_trace(output)
+        output = io.StringIO()
+        sim.tracer.print_trace(output)
         self.assertEqual(output.getvalue(), correct_output)
 
     def test_step_multiple_many_errors_report_only_first(self):
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
 
         expected = {
             'out1': [7, 9, 4, 10, 9],
             'out2': [6, 2, 7, 8, 14],
         }
-        output = six.StringIO()
+        output = io.StringIO()
         sim.step_multiple(self.inputs, expected, file=output, stop_after_first_error=True)
 
         # Test the output about unexpected values
@@ -730,19 +825,18 @@ class SimStepMultipleBase(unittest.TestCase):
                           "in2  6 6\n"
                           "out1 7 8\n"
                           "out2 6 7\n")
-        output = six.StringIO()
-        sim_trace.print_trace(output)
+        output = io.StringIO()
+        sim.tracer.print_trace(output)
         self.assertEqual(output.getvalue(), correct_output)
 
     def test_step_multiple_many_errors_report_all(self):
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
 
         expected = {
             'out1': [7, 9, 4, 10, 9],
             'out2': [6, 2, 7, 8, 14],
         }
-        output = six.StringIO()
+        output = io.StringIO()
         sim.step_multiple(self.inputs, expected, file=output)
 
         # Test the output about unexpected values
@@ -760,8 +854,8 @@ class SimStepMultipleBase(unittest.TestCase):
                           "in2   6  6  6  6  6\n"
                           "out1  7  8  6 10  9\n"
                           "out2  6  7  7 15 14\n")
-        output = six.StringIO()
-        sim_trace.print_trace(output)
+        output = io.StringIO()
+        sim.tracer.print_trace(output)
         self.assertEqual(output.getvalue(), correct_output)
 
 
@@ -774,19 +868,18 @@ class TraceWithAdderBase(unittest.TestCase):
         self.r.next <<= self.result
 
     def test_adder_simulation(self):
-        sim_trace = pyrtl.SimulationTrace()
         on_reset = {}  # signal states to be set when reset is asserted
         # build the actual simulation environment
-        sim = self.sim(register_value_map=on_reset, default_value=0, tracer=sim_trace)
+        sim = self.sim(register_value_map=on_reset, default_value=0)
 
         # step through 15 cycles
         for i in range(15):
-            sim.step({})
+            sim.step()
 
-        output = six.StringIO()
-        sim_trace.print_trace(output, compact=True)
-        file = six.StringIO()
-        sim_trace.render_trace(file=file)  # want to make sure the code at least runs
+        output = io.StringIO()
+        sim.tracer.print_trace(output, compact=True)
+        file = io.StringIO()
+        sim.tracer.render_trace(file=file)  # want to make sure the code at least runs
         self.assertEqual(output.getvalue(), 'r 012345670123456\n')
         self.assertEqual(sim.inspect(self.r), 6)
 
@@ -856,17 +949,16 @@ b110 r
 """
 
     def test_vcd_output(self):
-        sim_trace = pyrtl.SimulationTrace()
         on_reset = {}  # signal states to be set when reset is asserted
         # build the actual simulation environment
-        sim = self.sim(register_value_map=on_reset, default_value=0, tracer=sim_trace)
+        sim = self.sim(register_value_map=on_reset, default_value=0)
 
         # step through 15 cycles
         for i in range(15):
-            sim.step({})
+            sim.step()
 
-        test_output = six.StringIO()
-        sim_trace.print_vcd(test_output)
+        test_output = io.StringIO()
+        sim.tracer.print_vcd(test_output)
         self.assertEqual(self.VCD_OUTPUT, test_output.getvalue())
 
 
@@ -881,8 +973,7 @@ class SimTraceWithMuxBase(unittest.TestCase):
         self.muxout <<= pyrtl.mux(self.sel, self.a, self.b)
 
         # build the actual simulation environment
-        self.sim_trace = pyrtl.SimulationTrace()
-        self.sim = self.sim(tracer=self.sim_trace)
+        self.sim = self.sim()
 
     def test_adder_simulation(self):
         input_signals = {0: {self.a: 0, self.b: 1, self.sel: 1},
@@ -894,8 +985,8 @@ class SimTraceWithMuxBase(unittest.TestCase):
         for i in range(6):
             self.sim.step(input_signals[i])
 
-        output = six.StringIO()
-        self.sim_trace.print_trace(output, compact=True)
+        output = io.StringIO()
+        self.sim.tracer.print_trace(output, compact=True)
         self.assertEqual(output.getvalue(), 'muxout 120120\n')
 
 
@@ -917,11 +1008,8 @@ class MemBlockBase(unittest.TestCase):
         self.output2 <<= self.mem1[self.read_addr2]
         self.mem1[self.write_addr] <<= self.write_data
 
-        # build the actual simulation environment
-        self.sim_trace = pyrtl.SimulationTrace()
-
     def test_simple_memblock(self):
-        sim = self.sim(tracer=self.sim_trace)
+        sim = self.sim()
 
         input_signals = [[0, 1, 4, 5],
                          [4, 1, 0, 5],
@@ -932,12 +1020,12 @@ class MemBlockBase(unittest.TestCase):
             sim.step({self.read_addr1: signals[0], self.read_addr2: signals[1],
                       self.write_addr: signals[2], self.write_data: signals[3]})
 
-        output = six.StringIO()
-        self.sim_trace.print_trace(output, compact=True)
+        output = io.StringIO()
+        sim.tracer.print_trace(output, compact=True)
         self.assertEqual(output.getvalue(), 'o1 05560\no2 00560\n')
 
     def test_simple2_memblock(self):
-        sim = self.sim(tracer=self.sim_trace)
+        sim = self.sim()
         input_signals = [
             {self.read_addr1: 0, self.read_addr2: 1, self.write_addr: 0, self.write_data: 0x7},
             {self.read_addr1: 1, self.read_addr2: 2, self.write_addr: 1, self.write_data: 0x6},
@@ -953,15 +1041,14 @@ class MemBlockBase(unittest.TestCase):
         for signal in input_signals:
             sim.step(signal)
 
-        output = six.StringIO()
-        self.sim_trace.print_trace(output, compact=True)
+        output = io.StringIO()
+        sim.tracer.print_trace(output, compact=True)
         self.assertEqual(output.getvalue(), 'o1 0077653107\no2 0076452310\n')
 
     def test_synth_simple_memblock(self):
         pyrtl.synthesize()
         pyrtl.optimize()
-        self.sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=self.sim_trace)
+        sim = self.sim()
         input_signals = [[0, 1, 4, 5],
                          [4, 1, 0, 5],
                          [0, 4, 1, 6],
@@ -971,8 +1058,8 @@ class MemBlockBase(unittest.TestCase):
             sim.step({self.read_addr1: signals[0], self.read_addr2: signals[1],
                       self.write_addr: signals[2], self.write_data: signals[3]})
 
-        output = six.StringIO()
-        self.sim_trace.print_trace(output, compact=True)
+        output = io.StringIO()
+        sim.tracer.print_trace(output, compact=True)
         self.assertEqual(output.getvalue(), 'o1 05560\no2 00560\n')
 
     def test_mem_val_map(self):
@@ -981,8 +1068,7 @@ class MemBlockBase(unittest.TestCase):
         self.output3 <<= self.mem2[read_addr3]
         mem_val_map = {self.mem1: {0: 0, 1: 1, 2: 2, 3: 3},
                        self.mem2: {0: 4, 1: 5, 2: 6, 3: 7}}
-        self.sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=self.sim_trace, memory_value_map=mem_val_map)
+        sim = self.sim(memory_value_map=mem_val_map)
         # put new entries in
         for i in range(2):
             sim.step({
@@ -1010,8 +1096,7 @@ class MemBlockBase(unittest.TestCase):
         self.output3 <<= self.mem2[read_addr3]
         mem_val_map = {self.mem1: {0: 0, 1: 1},
                        self.mem2: {0: 4, 1: 5}}
-        self.sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=self.sim_trace, memory_value_map=mem_val_map)
+        sim = self.sim(memory_value_map=mem_val_map)
         for i in range(2, 8):
             sim.step({
                 self.read_addr1: i,
@@ -1020,8 +1105,8 @@ class MemBlockBase(unittest.TestCase):
                 self.write_addr: 0,
                 self.write_data: 0
             })
-        output = six.StringIO()
-        self.sim_trace.print_trace(output, compact=True)
+        output = io.StringIO()
+        sim.tracer.print_trace(output, compact=True)
         self.assertEqual(output.getvalue(), 'o1 000000\n'
                                             'o2 000000\n'
                                             'o3 000000\n')
@@ -1043,13 +1128,10 @@ class MemBlockLargeBase(unittest.TestCase):
         self.output2 <<= self.mem[self.read_addr2]
         self.mem[self.write_addr] <<= self.write_data
 
-        # build the actual simulation environment
-        self.sim_trace = pyrtl.SimulationTrace()
-
     def test_mem_blocks_very_large(self):
         ''' Tests support of very large memories (i.e. address width > 30 bits),
             and that limbs are handled appropriately for bitwidths > 64 '''
-        sim = self.sim(tracer=self.sim_trace)
+        sim = self.sim()
 
         write_data = 0x20000000040000012  # 68 bits
         input_signals = [[0, 1, 0xffffffff, write_data],
@@ -1061,12 +1143,12 @@ class MemBlockLargeBase(unittest.TestCase):
             sim.step({self.read_addr1: signals[0], self.read_addr2: signals[1],
                       self.write_addr: signals[2], self.write_data: signals[3]})
 
-        output = six.StringIO()
+        output = io.StringIO()
         correct_outp = ("--- Values in base 10 ---\n"
                         "o1                    0 %d %d                    6                    0\n"
                         "o2                    0                    0 %d                    6                    0\n"  # noqa
                         % (write_data, write_data, write_data))
-        self.sim_trace.print_trace(output)
+        sim.tracer.print_trace(output)
         self.assertEqual(output.getvalue(), correct_outp)
 
 
@@ -1082,12 +1164,11 @@ class RegisterDefaultsBase(unittest.TestCase):
         self.o <<= self.r2
 
     def check_trace(self, correct_string, **kwargs):
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace, **kwargs)
+        sim = self.sim(**kwargs)
         for i in range(8):
             sim.step({self.i: i})
-        output = six.StringIO()
-        sim_trace.print_trace(output, compact=True)
+        output = io.StringIO()
+        sim.tracer.print_trace(output, compact=True)
         self.assertEqual(output.getvalue(), correct_string)
 
     def test_default_value(self):
@@ -1124,7 +1205,7 @@ class RomBlockSimBase(unittest.TestCase):
         return out_string
 
     def compareIO(self, sim_trace_a, expected_output):
-        output = six.StringIO()
+        output = io.StringIO()
         sim_trace_a.print_trace(output, compact=True)
         self.assertEqual(output.getvalue(), expected_output)
 
@@ -1145,8 +1226,7 @@ class RomBlockSimBase(unittest.TestCase):
         self.output1 <<= self.rom[self.read_addr1]
         self.output2 <<= self.rom[self.read_addr2]
         # build the actual simulation environment
-        self.sim_trace = pyrtl.SimulationTrace()
-        self.sim = self.sim(tracer=self.sim_trace)
+        self.sim = self.sim()
 
         input_signals = {}
         for i in range(0, 5):
@@ -1155,7 +1235,7 @@ class RomBlockSimBase(unittest.TestCase):
 
         exp_out = self.generate_expected_output((("o1", lambda x: rom_data_function(x)),
                                                  ("o2", lambda x: rom_data_function(2 * x))), 6)
-        self.compareIO(self.sim_trace, exp_out)
+        self.compareIO(self.sim.tracer, exp_out)
 
     def test_function_RomBlock_with_optimization(self):
 
@@ -1178,8 +1258,7 @@ class RomBlockSimBase(unittest.TestCase):
         pyrtl.synthesize()
         pyrtl.optimize()
         # build the actual simulation environment
-        self.sim_trace = pyrtl.SimulationTrace()
-        self.sim = self.sim(tracer=self.sim_trace)
+        self.sim = self.sim()
 
         input_signals = {}
         for i in range(0, 5):
@@ -1190,7 +1269,7 @@ class RomBlockSimBase(unittest.TestCase):
         # exp_out = self.generate_expected_output((("o1", lambda x: rom_data_function(x) - 1),
         exp_out = self.generate_expected_output((("o1", lambda x: rom_data_function(x)),
                                                  ("o2", lambda x: rom_data_function(2 * x))), 6)
-        self.compareIO(self.sim_trace, exp_out)
+        self.compareIO(self.sim.tracer, exp_out)
 
     def test_rom_out_of_range_error(self):
         rom_data_array = [15, 13, 11, 9, 7, 5, 3]
@@ -1199,8 +1278,7 @@ class RomBlockSimBase(unittest.TestCase):
         rom_out_1 = pyrtl.Output(4, "rom_out_1")
         rom_out_1 <<= rom1[rom_add_1]
 
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
         sim.step({rom_add_1: 3})
         with self.assertRaises(pyrtl.PyrtlError):
             sim.step({rom_add_1: 7})
@@ -1218,9 +1296,8 @@ class RomBlockSimBase(unittest.TestCase):
         mem_val_map = {self.rom1: {0: 0, 1: 1, 2: 2, 3: 3},
                        self.rom2: {0: 4, 1: 5, 2: 6, 3: 7}}
 
-        self.sim_trace = pyrtl.SimulationTrace()
         with self.assertRaises(pyrtl.PyrtlError):
-            sim = self.sim(tracer=self.sim_trace, memory_value_map=mem_val_map)
+            sim = self.sim(memory_value_map=mem_val_map)
 
 
 class InspectBase(unittest.TestCase):
@@ -1233,8 +1310,7 @@ class InspectBase(unittest.TestCase):
 
     def test_invalid_inspect(self):
         a = pyrtl.Input(8, 'a')
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
         sim.step({a: 28})
         with self.assertRaises(KeyError):
             sim.inspect('asd')
@@ -1243,8 +1319,7 @@ class InspectBase(unittest.TestCase):
         a = pyrtl.Input(8, 'b')
         b = pyrtl.Output(name='a')
         b <<= a
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
         if self.sim is pyrtl.Simulation:
             self.assertEqual(sim.inspect(a), 0)
             self.assertEqual(sim.inspect(b), 0)
@@ -1262,8 +1337,7 @@ class InspectBase(unittest.TestCase):
         b = pyrtl.Input(8, 'b')
         mem = pyrtl.MemBlock(8, 8, 'mem')
         mem[b] <<= a
-        sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=sim_trace)
+        sim = self.sim()
         self.assertEqual(sim.inspect_mem(mem), {})
         sim.step({a: 3, b: 23})
         self.assertEqual(sim.inspect_mem(mem), {23: 3})
@@ -1275,18 +1349,36 @@ class TraceErrorBase(unittest.TestCase):
 
     def test_empty_trace(self):
         with self.assertRaises(pyrtl.PyrtlError):
-            self.sim_trace = pyrtl.SimulationTrace()
+            sim_trace = pyrtl.SimulationTrace()
 
     def test_invalid_base(self):
         self.in1 = pyrtl.Input(8, "in1")
         self.out = pyrtl.Output(8, "out")
         self.out <<= self.in1
-        self.sim_trace = pyrtl.SimulationTrace()
-        sim = self.sim(tracer=self.sim_trace)
+        sim = self.sim()
         for i in range(5):
             sim.step({self.in1: i})
         with self.assertRaises(pyrtl.PyrtlError):
-            self.sim_trace.print_trace(base=4)
+            sim.tracer.print_trace(base=4)
+
+
+class NoTracerBase(unittest.TestCase):
+    """Test simulation without a SimulationTrace."""
+    def setUp(self):
+        pyrtl.reset_working_block()
+
+    def test_no_tracer(self):
+        a_val = 1
+        b_val = 2
+        expected_c = a_val + b_val
+        a = pyrtl.Const(a_val, bitwidth=2)
+        b = pyrtl.Const(b_val, bitwidth=2)
+        c = pyrtl.Output(name='c', bitwidth=3)
+        c <<= a + b
+        sim = self.sim(tracer=None)
+        sim.step()
+        actual_c = sim.inspect('c')
+        self.assertEqual(expected_c, actual_c)
 
 
 def make_unittests():
